@@ -2,8 +2,8 @@ use std::str;
 
 use tree_sitter::{InputEdit, Parser, Point, Range, Tree};
 
-use super::helpers::{edits::invert_edit, fixtures::get_language};
-use crate::parse::{perform_edit, Edit};
+use super::helpers::fixtures::get_language;
+use crate::{fuzz::edits::Edit, parse::perform_edit, tests::invert_edit};
 
 #[test]
 fn test_tree_edit() {
@@ -679,6 +679,56 @@ fn test_get_changed_ranges() {
     }
 }
 
+#[test]
+fn test_consistency_with_mid_codepoint_edit() {
+    let mut parser = Parser::new();
+    parser.set_language(&get_language("php/php")).unwrap();
+    let mut source_code =
+        b"\n<?php\n\n<<<'\xE5\xAD\x97\xE6\xBC\xA2'\n  T\n\xE5\xAD\x97\xE6\xBC\xA2;".to_vec();
+    let mut tree = parser.parse(&source_code, None).unwrap();
+
+    let edit = Edit {
+        position: 17,
+        deleted_length: 0,
+        inserted_text: vec![46],
+    };
+    perform_edit(&mut tree, &mut source_code, &edit).unwrap();
+    let mut tree2 = parser.parse(&source_code, Some(&tree)).unwrap();
+
+    let inverted = invert_edit(&source_code, &edit);
+    perform_edit(&mut tree2, &mut source_code, &inverted).unwrap();
+    let tree3 = parser.parse(&source_code, Some(&tree2)).unwrap();
+
+    assert_eq!(tree3.root_node().to_sexp(), tree.root_node().to_sexp());
+}
+
+#[test]
+fn test_tree_cursor_on_aliased_root_with_extra_child() {
+    let source = r#"
+fn main() {
+    C/* hi */::<D>::E;
+}
+"#;
+
+    let mut parser = Parser::new();
+    parser.set_language(&get_language("rust")).unwrap();
+
+    let tree = parser.parse(source, None).unwrap();
+
+    let function = tree.root_node().child(0).unwrap();
+    let block = function.child(3).unwrap();
+    let expression_statement = block.child(1).unwrap();
+    let scoped_identifier = expression_statement.child(0).unwrap();
+    let generic_type = scoped_identifier.child(0).unwrap();
+    assert_eq!(generic_type.kind(), "generic_type");
+
+    let mut cursor = generic_type.walk();
+    assert!(cursor.goto_first_child());
+    assert_eq!(cursor.node().kind(), "type_identifier");
+    assert!(cursor.goto_next_sibling());
+    assert_eq!(cursor.node().kind(), "block_comment");
+}
+
 fn index_of(text: &[u8], substring: &str) -> usize {
     str::from_utf8(text).unwrap().find(substring).unwrap()
 }
@@ -701,7 +751,7 @@ fn get_changed_ranges(
     edit: &Edit,
 ) -> Vec<Range> {
     perform_edit(tree, source_code, edit).unwrap();
-    let new_tree = parser.parse(&source_code, Some(tree)).unwrap();
+    let new_tree = parser.parse(source_code, Some(tree)).unwrap();
     let result = tree.changed_ranges(&new_tree).collect();
     *tree = new_tree;
     result
